import 'dart:async';

import 'package:flutter/material.dart';
import 'package:flutter/rendering.dart';

import './input_helper.dart';

typedef CaretMoved = void Function(Offset globalCaretPosition);
typedef TextChanged = void Function(String text);

class CustomTextInput extends StatefulWidget {
  const CustomTextInput(
      {required Key key,
      required this.onCaretMoved,
      required this.onTextChanged,
      required this.enable,
      required this.label,
      required this.icon,
      required this.focusChange,
      this.isObscured = false})
      : super(key: key);
  final CaretMoved onCaretMoved;
  final TextChanged onTextChanged;
  final Function(bool) focusChange;
  final String label;
  final bool isObscured;
  final bool enable;

  final IconData icon;

  @override
  _TrackingTextInputState createState() => _TrackingTextInputState();
}

class _TrackingTextInputState extends State<CustomTextInput> {
  final GlobalKey _fieldKey = GlobalKey();
  final TextEditingController _textController = TextEditingController();
  Timer? _debounceTimer;
  @override
  initState() {
    _textController.addListener(() {
      if (_debounceTimer?.isActive ?? false) _debounceTimer!.cancel();
      _debounceTimer = Timer(const Duration(milliseconds: 100), () {
        if (_fieldKey.currentContext != null) {
          final RenderObject? fieldBox =
              _fieldKey.currentContext!.findRenderObject();
          Offset caretPosition = getCaretPosition(fieldBox!);

          if (widget.onCaretMoved != null) {
            widget.onCaretMoved(caretPosition);
          }
        }
      });
      if (widget.onTextChanged != null) {
        widget.onTextChanged(_textController.text);
      }
    });
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return Focus(
      onFocusChange: widget.focusChange,
      child: TextFormField(
        key: _fieldKey,
        style: const TextStyle(fontSize: 16.0),
        enabled: widget.enable,
        keyboardType:
            widget.isObscured ? TextInputType.text : TextInputType.emailAddress,
        textInputAction: TextInputAction.done,
        decoration: InputDecoration(
          labelText: widget.label,
          labelStyle: const TextStyle(fontSize: 16.0),
          prefixIcon: Icon(widget.icon),
        ),
        controller: _textController,
        obscureText: widget.isObscured,
      ),
    );
  }
}
